searchNodes=[{"doc":"Queue-free/gen_server-free/process-free locking mechanism built for high concurrency. This allows for the use of locks in hot-code-paths without being bottlenecked by processes message queues. Usage Simply call the transaction/2,3,4 function with a locking key, and a function to apply, the function is guaranteed to be excluded while exclusively locking the given key. Examples Given two functions, one being slower than the other, the ordering isn't guaranteed if both are run by separate process, and if they have some side effects, such as update cache values, this could be undesirable. iex&gt; myself = self ( ) iex&gt; first_function = fn -&gt; ...&gt; :timer . sleep ( 2_000 ) ...&gt; send myself , :first_function ...&gt; end iex&gt; second_function = fn -&gt; ...&gt; :timer . sleep ( 1_000 ) ...&gt; send myself , :second_function ...&gt; end iex&gt; spawn first_function iex&gt; spawn second_function iex&gt; :ok = receive do ...&gt; :first_function -&gt; :error ...&gt; :second_function -&gt; :ok ...&gt; end iex&gt; :ok = receive do ...&gt; :first_function -&gt; :ok ...&gt; :second_function -&gt; :error ...&gt; end Using Elixir.Locksmith you can ensure they don't run at the same time by locking both under the same key, note that ordering isn't guaranteed per-se as the lock is given on a first-come-first-serve bases. iex&gt; myself = self ( ) iex&gt; first_function = fn -&gt; ...&gt; :timer . sleep ( 2_000 ) ...&gt; send myself , :first_function ...&gt; end iex&gt; second_function = fn -&gt; ...&gt; :timer . sleep ( 1_000 ) ...&gt; send myself , :second_function ...&gt; end iex&gt; spawn fn -&gt; Locksmith . transaction ( &quot;somekey&quot; , first_function ) end iex&gt; spawn fn -&gt; Locksmith . transaction ( &quot;somekey&quot; , second_function ) end iex&gt; :ok = receive do ...&gt; :first_function -&gt; :ok ...&gt; :second_function -&gt; :error ...&gt; end iex&gt; :ok = receive do ...&gt; :first_function -&gt; :error ...&gt; :second_function -&gt; :ok ...&gt; end Notice that after using Locksmith the first transaction which acquired the lock first managed to execute even before the second transaction could start, since the second transaction was locked until the key is released. Implementation When starting this module, it'll initiate an Eternal process, which will handle creating and maintaining a long live ETS table that is not bound by any process and lives across the applications lifecycle. This is required since ETS tables are bound by the process that creates them and when the process dies the ETS table is deleted unless their an &quot;heir&quot; to the table, Eternal handles retaining the table as long as our app lives. Internally this module utilizes :ets.update_counter/4 function, which provides us with an atomic and isolated updates to counter in ETS tables. Given any transaction with a lock key, a lock is &quot;acquired&quot;, this is achieved by calling :ets.update_counter/4 , defaulting the lock key to counter of value 0 if not found and then incrementing it by 1 , if the counter after update is equal to 1 then the lock is acquired otherwise the lock isn't acquired. The increment operation is done with a threshold and set_value set to 2 forcing the counter to never exceed the value 2 . This means each lock counter is set to a three state value, 0 lock is free, 1 lock has been acquired, 2 lock was acquired by someone else. After the function is applied the lock is &quot;released&quot; by resetting it's value to 0 , this is done by increment by 1 , while having threshold and set_value to 0 each, forcing the counter to reset atomically. If the acquire operation fails (returns false ), the current process is blocked via a receive/1 operation, and before running the receive/1 operation it'll sent itself a delayed message using Process.send/4 , once it receives the message it sent itself, it'll attempt to acquire the lock again, this behaviour is done recursively.","ref":"Locksmith.html","title":"Locksmith","type":"module"},{"doc":"Given a locking key, an anonymous function of arity zero, lock the given key and execute the function, then release the key. If the key is already locked then retry to lock the key and run again after some delay. This is achieved by blocking the caller process using a receive/1 call coupled with Process.send_after/4 .","ref":"Locksmith.html#transaction/2","title":"Locksmith.transaction/2","type":"function"},{"doc":"Alternative to transaction/2 that takes an anonymous function of any arity and a list of arguments for it.","ref":"Locksmith.html#transaction/3","title":"Locksmith.transaction/3","type":"function"},{"doc":"Alternative to transaction/2 that takes an MFA instead of an anonymous function.","ref":"Locksmith.html#transaction/4","title":"Locksmith.transaction/4","type":"function"},{"doc":"Locksmith Queue-free/gen_server-free/process-free locking mechanism built for high concurrency. In certain scenarios you may require to have locking on resources in hot code paths, most locking solutions require some singular process that, under heavy loads, this may cause bottlenecking on that single process. Locksmith attempts to avoid this by avoiding all forms of processes to avoid all kinds of bottlenecks in your hot code paths. This is achieved by use of ets:update_counter/4 operations, (ab)using it's atomicity and isolation.","ref":"readme.html","title":"Locksmith","type":"extras"},{"doc":"The package can be installed by adding locksmith to your list of dependencies in mix.exs : def deps do [ { :locksmith , &quot;~&gt; 1.0.0&quot; } ] end","ref":"readme.html#installation","title":"Locksmith - Installation","type":"extras"}]